 /////////////////////////////////////////////////////////////////////////////////////////
// Module Name: FW_2200.ino
// Project: Foucault Pendulum, Subsystem BobControl
// Target: Arduino MEGA + ETHernet Shield
// Last Mod: See VERSIONSTRING
// jan@breem.nl,  janbee@hack42.nl, mail@foucaultpendulum.nl
// www.foucaultslinger.nl   www.foucaultpendulum.nl
//
/////////////////////////////////////////////////////////////////////////////////////////
#include "Globals.h"
#include <Arduino.h>
#include <Adafruit_INA219.h>  // for High-Side Current / Voltage Sensor

#define VERSIONSTRING "BobControl 2022-12-25"
#define VERSIONNUMBER 1225 

unsigned int VersionNumber;
bool GeneralError;

// for power management
Adafruit_INA219 Ina219; 
int BatteryVoltage_mV, BatteryCurrent_mA;
bool DoReadBatteryStatus;

/**************************************************************************************************/
byte EEPROM_ReadByte (unsigned int EEpromAddress)
{
  // Wait for completion of previous action
  while (EECR & (1 << EEPE));
  EEAR = EEpromAddress;
  EECR |= (1 << EERE);
  return EEDR;
}

/**************************************************************************************************/
int EEPROM_ReadInt(unsigned int EEpromAddress)
{
  int lo, hi;
  lo = EEPROM_ReadByte (EEpromAddress);
  hi = EEPROM_ReadByte (EEpromAddress + 1);
  return ((hi << 8) | lo);
}

/**************************************************************************************************/
void EEPROM_WriteByte (unsigned int EEpromAddress, byte EEpromData)
{
  // Wait for completion of previous action
  while (EECR & (1 << EEPE));
  EEAR = EEpromAddress;
  EEDR = EEpromData;
  EECR |= (1 << EEMPE);
  EECR |= (1 << EEPE);
}

/**************************************************************************************************/
void EEPROM_WriteInt (unsigned int EEpromAddress, int EEpromData)
{
  EEPROM_WriteByte (EEpromAddress, lowByte (EEpromData));
  EEPROM_WriteByte (EEpromAddress + 1, highByte (EEpromData));
}

/**************************************************************************************************/
void EEPROM_Clear (void)
{
  for (byte i=0; i<30; i++) EEPROM_WriteByte (i, 255); // looks as never written to
}

/**************************************************************************************************/
void WriteParametersToEEprom (int StartAddress)
{
  DoWriteParametersToEEprom = false;
  Serial.println ("Write Parameters to EEPROM");
  EEPROM_WriteInt (StartAddress     , TStartLookForCenter);
  EEPROM_WriteInt (StartAddress +  2, TMissedCenter);
  EEPROM_WriteInt (StartAddress +  4, TStartLookForRim_1);  
  EEPROM_WriteInt (StartAddress +  6, TMissedRim_1);
  EEPROM_WriteInt (StartAddress +  8, TStartLookForRim_2);
  EEPROM_WriteInt (StartAddress + 10, TMissedRim_2);
  EEPROM_WriteInt (StartAddress + 12, RimCoilRadius_mm);
  EEPROM_WriteInt (StartAddress + 14, SetPoint_Amplitude_mm);

  EEPROM_WriteInt (StartAddress + 16, TMidDrive);
  EEPROM_WriteInt (StartAddress + 18, TMinimalDriveWidth);
  EEPROM_WriteInt (StartAddress + 20, TMaximalDriveWidth);
}

/**************************************************************************************************/
void ReadParametersFromEEprom (int StartAddress)
{
  TStartLookForCenter =   EEPROM_ReadInt (StartAddress);
  TMissedCenter =         EEPROM_ReadInt (StartAddress +  2);
  TStartLookForRim_1 =    EEPROM_ReadInt (StartAddress +  4);
  TMissedRim_1 =          EEPROM_ReadInt (StartAddress +  6);
  TStartLookForRim_2 =    EEPROM_ReadInt (StartAddress +  8);
  TMissedRim_2 =          EEPROM_ReadInt (StartAddress + 10);
  RimCoilRadius_mm =      EEPROM_ReadInt (StartAddress + 12);
  SetPoint_Amplitude_mm = EEPROM_ReadInt (StartAddress + 14); 
  TMidDrive =             EEPROM_ReadInt (StartAddress + 16);
  TMinimalDriveWidth =    EEPROM_ReadInt (StartAddress + 18);
  TMaximalDriveWidth =    EEPROM_ReadInt (StartAddress + 20);
  Serial.println ("Done Reading from EEPROM");
  DoReportParameters = true;
}

//********************************************************************************************************
void ReportParameters(void)
{
  DoReportParameters = false; // only once
  Serial.println ("\nSettings for synchronisation");
  Serial.print ("TStartLookForCenter   "); Serial.println (TStartLookForCenter);
  Serial.print ("TMissedCenter         "); Serial.println (TMissedCenter);  
  Serial.print ("TStartLookForRim_1    "); Serial.println (TStartLookForRim_1); 
  Serial.print ("TMissedRim_1          "); Serial.println (TMissedRim_1); 
  Serial.print ("TStartLookForRim_21   "); Serial.println (TStartLookForRim_2); 
  Serial.print ("TMissedRim_2          "); Serial.println (TMissedRim_2); 
  Serial.println ("\nSettings for Drive:");
  Serial.print ("TMinimalDriveWidth    "); Serial.println (TMinimalDriveWidth); 
  Serial.print ("TMaximalDriveWidth    "); Serial.println (TMaximalDriveWidth); 
  Serial.print ("TMidDrive             "); Serial.println (TMidDrive); 
  Serial.print ("TStartDrive           "); Serial.println (TStartDrive); 
  Serial.print ("TStopDrive            "); Serial.println (TStopDrive);  
  Serial.println ("\nSettings for Amplitude Control:");
  Serial.print ("RimCoilRadius mm      "); Serial.println (RimCoilRadius_mm); 
  Serial.print ("Setpoint Amplitude mm "); Serial.println (SetPoint_Amplitude_mm); 
  Serial.println ();
}

//********************************************************************************************************
void ResetArduino (byte ResetCause)
{
  // Resetting the Arduino from software can be done by jumping to the reset vector, but
  // then the hardware will not be reset, particularly the Ethernet Shield.
  // Here we pull the RESET line LOW with a Fet, causing a complete hardware reset.
  // This requires a special circuit to stretch and invert the reset pulse. See diagram.
  // But first we write something to EEPROM, so we can find out what caused the reset.
  EEPROM_WriteByte (RESET_FLAG_ADDRESS, ResetCause);
  delay (100);
  DDRH |= 0x20;  // Data direction = Output
  PORTH |= 0x20; // PORTH bit 5 pin 8 high brings the FET in conduction after some delay.
}

//********************************************************************************************************
void ReadBatteryStatus (void) 
{
  DIAGPIN_A9_H
  DoReadBatteryStatus = false;
  BatteryVoltage_mV = round (Ina219.getBusVoltage_V() * 1000);
  BatteryCurrent_mA = -round (Ina219.getCurrent_mA()); // Show charging Positive
  DIAGPIN_A9_L
}  

//********************************************************************************************************
void ShowBatteryState (void) 
{
  ReadBatteryStatus ();
  Serial.print("Battery Voltage: "); Serial.print (BatteryVoltage_mV); Serial.print (" mV");
  Serial.print(" Charge Current: "); Serial.print (BatteryCurrent_mA); Serial.println (" mA");
  Serial.println("");
}

//********************************************************************************************************
void ShowHelp (void) 
{
  Serial.println ("\nAvailable commands: \n");
  Serial.println ("b: Show Battery State");
  Serial.println ("C: Clear EEPROM Area");
  Serial.println ("D: Set Default Parameters");
  Serial.println ("H: Show this Help info");
  Serial.println ("p: Show Parameters");
  Serial.println ("P: Read Parameters from EEPROM"); 
  Serial.println ("R: RESET ARDUINO");
}

//********************************************************************************************************
void setup(void)
{
  Serial.begin (19200);
  Serial.println ("\n\n\n\n\n\n\n\n\n");
  Serial.println (VERSIONSTRING);
  VersionNumber = VERSIONNUMBER;
  Serial.print ("Version Number: ");
  Serial.println (VersionNumber);  
  Serial.println();

  // Find out if we woke up from a self induced reset
  byte ResetCause = EEPROM_ReadByte (RESET_FLAG_ADDRESS);
  Serial.print ("Reset Cause: ");
  Serial.println (ResetCause);
  if (ResetCause == 0) DidResetMyself = true;
  EEPROM_WriteByte (RESET_FLAG_ADDRESS, 255); // now it looks as never touched
  
  // I/O's
  DDRA |= 0x05;  // HOLD and CHRG
  PORTA|= 0xAA;  // Activate Internal Pullup's for Jumper Block.

  CHARGEBATTERY_ON
  HOLDPOWER_ON

  ReadParametersFromEEprom (EEPROMSTARTADDRESS);
  ReportParameters ();
    
  Init_BobControl ();
  Init_Messages ();

  // Initialize the high-side current sensor INA219
  // By default the initialization will use the largest range (32V, 2A).
  if (!Ina219.begin()) Serial.println ("Failed to find INA219 chip");
  // Use a lower 16V, 400mA range (higher precision on volts and amps):
  Ina219.setCalibration_16V_400mA();
  
  USEDIAGPIN_A8  // Message in / out
  USEDIAGPIN_A9  // Reading battery voltage and current
  USEDIAGPIN_A10
  USEDIAGPIN_A11
  USEDIAGPIN_A12 // PassRim_2
  USEDIAGPIN_A13 // PassRim_1
  USEDIAGPIN_A14 // PassCenter
  USEDIAGPIN_A15 // 10 kHz Timer 1 interrupt handler InService time
}

// ********************************************************************************************************
void loop(void)
{
  Update_Messages (); 
 
  char SerIn[2];
  if (Serial.available())
  {
    Serial.readBytes (SerIn, 2);
    if (SerIn[0] == 'b') ShowBatteryState ();
    if (SerIn[0] == 'C') EEPROM_Clear ();
    if (SerIn[0] == 'h') ShowHelp ();
    if (SerIn[0] == 'P') ReadParametersFromEEprom (EEPROMSTARTADDRESS);
    if (SerIn[0] == 'p') DoReportParameters = true;
    if (SerIn[0] == 'R') ResetArduino (2);
  } 

  if (DoReportParameters) ReportParameters();
  if (DoWriteParametersToEEprom) WriteParametersToEEprom (EEPROMSTARTADDRESS);
  if (DoUpdateAmplitudeControl) UpdateAmplitudeControl();
  if (DoReadBatteryStatus) ReadBatteryStatus(); 
  
  if (false)  // to check proper working of A/D conversions
  {
    char Str[40];
    sprintf (Str, "%3d %3d %3d %3d   %3d %3d",
    AdcHallWest, AdcHallEast, AdcHallNorth, AdcHallSouth, AdcCenter, AdcRim);
    Serial.println (Str);
    delay (100);
  }  
}
